Exception: JSI::Schema::NotASchemaError
- Inherits:
-
TypeError
- Object
- TypeError
- JSI::Schema::NotASchemaError
- Defined in:
- lib/jsi/schema.rb
Overview
an exception raised when a thing is expected to be a JSI::Schema, but is not
an exception raised when a thing is expected to be a JSI::Schema, but is not